import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="neopixel",  # This is the name of the package
    version="0.0.1",  # The initial release version
    author="Ajat Prabha",  # Full name of the author
    description="An importable package to control LEDs via UDP protocol",
    license='MIT',
    url="https://github.com/ajatprabha/neopixel",
    platforms=['any'],
    long_description=long_description,  # Long description read from the the readme file
    long_description_content_type="text/markdown",
    packages=['neopixel'],  # List of all python modules to be installed
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],  # Information to filter the project on PyPi website
    python_requires='>=3.6',  # Minimum version requirement of the package
    py_modules=['neopixel'],  # Name of the python package
    package_dir={'neopixel': 'src'},  # Directory of the source code of the package
    install_requires=['numpy', 'scipy', 'pyaudio']  # Install other dependencies if any
)
